//+------------------------------------------------------------------+
//|                                                  Force Histo.mq4 |
//|                                                              cja |
//+------------------------------------------------------------------+
#property copyright "cja"
#property link      "ccjjaa@gmail.com"

#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1 Gray
#property indicator_color2 DeepSkyBlue
#property indicator_color3 SkyBlue
#property indicator_color4 PaleVioletRed
#property indicator_color5 DarkSalmon
#property indicator_color6 Silver
#property indicator_style1 STYLE_DOT
#property indicator_width2 2
#property indicator_width4 2
#property indicator_width6 2

//---- input parameters

extern string TimeFrame    = "Current time frame";
extern int	  Force_Period	= 5;
extern int    Force_Method	= 0;
extern int    Force_Price	= 0;
extern bool   Interpolate  = true;

string name;
         
//---- indicator buffers
double Force[];
double forceUP[];
double forceUPd[];
double forceDN[];
double forceDNu[];
double zero[];
double trend[];

int    timeFrame;
string indicatorFileName;
bool   returnBars;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   IndicatorBuffers(7);
   SetIndexBuffer(0,zero);
   SetIndexBuffer(1,forceUP);  SetIndexDrawBegin(1,Force_Period);  SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,forceUPd); SetIndexDrawBegin(2,Force_Period);  SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3,forceDN);  SetIndexDrawBegin(3,Force_Period);  SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(4,forceDNu); SetIndexDrawBegin(4,Force_Period);  SetIndexStyle(4,DRAW_HISTOGRAM);
   SetIndexBuffer(5,Force);    SetIndexDrawBegin(5,Force_Period);
   SetIndexBuffer(6,trend);

   SetIndexLabel(0,"zero line");
   SetIndexLabel(1,"Force Index");
   SetIndexLabel(2,"Force Index");
   SetIndexLabel(3,"Force Index");
   SetIndexLabel(4,"Force Index");
   SetIndexLabel(5,"Force Index");

      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame=="returnBars";     if (returnBars) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
      name              = (timeFrameToString(timeFrame)+" Force Histo");
   IndicatorShortName(name); 
   return(0);
}
int deinit() { return(0); }  

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int start()
{
   int i,limit,counted_bars=IndicatorCounted();

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { zero[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   
   if (timeFrame>Period()) limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for( i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);

      Force[i]    = iForce(NULL,timeFrame,Force_Period,Force_Method,Force_Price,y); 
      forceUP[i]  = EMPTY_VALUE;
      forceUPd[i] = EMPTY_VALUE;
      forceDN[i]  = EMPTY_VALUE;
      forceDNu[i] = EMPTY_VALUE;
      trend[i]    = trend[i+1];
      zero[i]     = 0;
         
         if(Force[i]>0) 
            {
               if (Force[i]>Force[i+1]) trend[i]=  10;
               if (Force[i]<Force[i+1]) trend[i]=  11;
            }
         if(Force[i]<0) 
            {
               if (Force[i]>Force[i+1]) trend[i]= -11;
               if (Force[i]<Force[i+1]) trend[i]= -10;
            }
         arangeHisto(i);
      
         if (timeFrame <= Period() || !Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

      //
      //
      //
      //
      //

      datetime time = iTime(NULL,timeFrame,y);
         for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
         for(int k = 1; k < n; k++)
         {
            Force[i+k] = Force[i] + (Force[i+n] - Force[i])*k/n;
               arangeHisto(i+k);
         }               
   }
   return(0);
}
  
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void arangeHisto(int i)
{
   if (trend[i] ==  10) forceUP[i]  = Force[i];
   if (trend[i] ==  11) forceUPd[i] = Force[i];
   if (trend[i] == -10) forceDN[i]  = Force[i];
   if (trend[i] == -11) forceDNu[i] = Force[i];
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}